/*Simple daemon logger for RSBAC
 ***************************************************
  You must change capabilities list for this program 
  to grand operations with RSBAC log
 ***************************************************
  rklog (c) - Stanislav I. Ievlev, 2000-2001
  RSBAC (c) - Amon Ott., 1995-2001
  Some part of code from klogd - (c) 1995  Martin Schulze <Martin.Schulze@Linux.DE>
  Some changes made by Amon Ott, 2000
 -----------------------------------------------------
*/

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <time.h>
#include <string.h>
#include <pwd.h>
#include <signal.h>
#include <syslog.h>
#include <rsbac/syscalls.h>
#include <errno.h>
extern int errno;

#include "pidfile.h"

char	*hostname=NULL;
struct sockaddr_in remote_server;
int	use_proc=1;

#define INPUT_LOG	"/proc/rsbac-info/rmsg"
#define OUTPUT_LOG	"/security-log"
#define SECOFF_UID	400
#define	PATH_VARRUN	"/var/run/"
static char	*pidfile=PATH_VARRUN "rklogd.pid";

#define MAX_LINE	8100
#define PORTNUM		1500

/*#define TEST*/

int check_father(pid_t father){
    if (kill(father, 0) && errno == ESRCH){
#ifdef TEST
	    puts("father dead");
#endif
                return(0);
    }
#ifdef TEST
    puts("father live");
#endif
    return 1;
}

void stop_work(int sig)
{
    syslog(LOG_INFO,"security kernel logger stopped");
	
#ifdef TEST
    puts("exiting...");
#endif

#ifndef TEST
    remove_pid(pidfile);
#endif
    if (!use_proc){
	char tmpstring[0];
	rsbac_log(0,tmpstring,sizeof(tmpstring));/*close kernel log buffer*/
    }
    if (sig==9) exit (1);
    exit(0);
}

char *getlogname(uid_t secoffuid)
{
    char *tmpstring=malloc(MAX_LINE);
    struct passwd *pwd;
    pwd=getpwuid(secoffuid);
    if (pwd!=NULL){
	strncpy(tmpstring,pwd->pw_dir,MAX_LINE);
	strncat(tmpstring,"/log/",MAX_LINE);
        strncat(tmpstring,OUTPUT_LOG,MAX_LINE);
    }else{
	fprintf(stderr,"wrong user \n");
	exit(1);
    }
	return tmpstring;
}

void open_out_net(char *servername)
{
    struct hostent *hp;

    if((hp=gethostbyname(servername))==0)
    {
	perror("cannot resolve hostname");
	exit(1);
    }

    bzero(&remote_server, sizeof(remote_server));
    bcopy(hp->h_addr,&remote_server.sin_addr,hp->h_length);

    remote_server.sin_family=hp->h_addrtype;
    remote_server.sin_port=htons(PORTNUM);
}

void write_net(char *data)
{
    struct sockaddr_in	clnt_addr;
    int			fd;
    int			count;
    
#ifdef TEST
    puts("writing to net...");
#endif

    if((fd=socket(AF_INET, SOCK_DGRAM, 0))==-1){
#ifdef TEST
	perror("open output socket");
	exit(1);
#else
	syslog(LOG_WARNING,"cannot open socket to send messages to remote host");
#endif
    }

    bzero(&clnt_addr, sizeof(clnt_addr));
    clnt_addr.sin_family=AF_INET;
    clnt_addr.sin_addr.s_addr=INADDR_ANY;

    if ( bind(fd, (struct sockaddr *)&clnt_addr, sizeof(clnt_addr))==-1){
#ifdef TEST
	perror("binding output socket");
	exit(1);
#else
	syslog(LOG_WARNING,"cannot bind socket to send messages to remote host");
#endif
    }

    count=sendto(fd,data,strlen(data),0,(struct sockaddr*)&remote_server,sizeof(remote_server));
#ifndef TEST
    if (count<=0){
	syslog(LOG_WARNING,"error during sending messages to remote host");
    }
#endif
}


int open_in(){
    int fd;
    if (use_proc){
	if ((fd=open(INPUT_LOG,O_RDONLY))==-1){
	    perror("open of proc file");
	    exit(1);
	}
    }else{
	char tmpstring[0];
	if (rsbac_log(1,tmpstring,sizeof(tmpstring)) <0)/*open kernel log buffer*/
	{
	    perror("open of kernel log buffer");
	    exit(1);
	};
    }
    return fd;
}

int open_out(char *name){
    int fd;
    if ((fd=open(name,O_WRONLY|O_APPEND|O_CREAT))==-1){
	perror("open of output log failed");
	exit(1);
    }
    fchmod(fd,0600);

    return fd;
}

int open_in_net()
{
    int fd;
    struct sockaddr_in serv_addr;
    
    if((fd=socket(AF_INET, SOCK_DGRAM, 0))==-1){
	perror("open of network socket failed");
	exit(1);
    }

    bzero(&serv_addr, sizeof(serv_addr));
    serv_addr.sin_family=AF_INET;
    serv_addr.sin_addr.s_addr=INADDR_ANY;
    serv_addr.sin_port=htons(PORTNUM);
    
    if( bind(fd, (struct sockaddr *)&serv_addr, sizeof(serv_addr))==-1){
	perror("bind failed");
	exit(1);
    }
    return fd;
}

void write_out(int fd,char* prefix,char *buf,int count){
    char *point=buf;
    char tmpstr[MAX_LINE];
    char *found;

    /*lock logfile*/
    if (flock(fd,LOCK_EX)<0){
#ifdef TEST
	perror("locking log file");
	exit(1);
#else
	syslog(LOG_WARNING,"cannot lock output file");
#endif
    }
    
#ifdef TEST
    puts("writing to log ...");
#endif

    do{
	found=strchr(point,'\n');
	if (found!=NULL){ 
	    strncpy(tmpstr,point,(size_t)(found-point)+1);
	    tmpstr[found-point+1]=0;
		write(fd,prefix,strlen(prefix)); 
		write(fd,tmpstr,found-point+1);
		if (hostname!=NULL) write_net(tmpstr);
	    found++;
	    count-=(found-point);
	}else{//
	    if (count>0){
		strncpy(tmpstr,point,count);
		tmpstr[count]=0;
		    write(fd,prefix,strlen(prefix));
		    write(fd,tmpstr,count);
		    if (hostname!=NULL) write_net(tmpstr);
	    }
	}
	point=found;
    }while(point!=NULL);

    /*ulock logfile*/
    if (flock(fd,LOCK_UN)<0){
#ifdef TEST
	perror("unlocking log file");
	exit(1);
#else
	syslog(LOG_WARNING,"cannot unlock output file");
#endif
    }
    
}
 
void read_in(int fd,int out){
    char	buf[MAX_LINE];
    char	prefix[MAX_LINE];
    size_t	count;

    time_t	now=time(0);
    char	*timestamp=ctime(&now);
    
    memset(buf,0,sizeof(buf));
    memset(prefix,0,sizeof(prefix));
    
#ifdef TEST
    puts("reading log...");
#endif
    if (use_proc){
	count=read(fd,buf,sizeof(buf));/*read from proc*/
    }else{
	count=rsbac_log(2,buf,sizeof(buf));/*read from log kernel buffer*/
    }
    
    strncpy(prefix,timestamp,strlen(timestamp)-1);
    strncat(prefix," :",3);
    write_out(out,prefix,buf,count);
}

void read_net(int fd,int out)
{
    struct sockaddr_in	clnt_addr;
    int			maxaddrlen=sizeof(clnt_addr);
    int			count;
    char		read_buf[MAX_LINE];
    char		log_buf[MAX_LINE];
    char		prefix[MAX_LINE];
    
    time_t		now=time(0);
    char		*timestamp=ctime(&now);

    memset(read_buf,0,sizeof(read_buf));
    memset(log_buf,0,sizeof(log_buf));
    memset(prefix,0,sizeof(prefix));

#ifdef TEST
    puts("reading net ...");
#endif

    count=recvfrom(fd,read_buf,sizeof(read_buf),0,(struct sockaddr *)&clnt_addr,&maxaddrlen);
    snprintf(log_buf,MAX_LINE,"from host %s :%s",inet_ntoa(clnt_addr.sin_addr),read_buf);
    
    strncpy(prefix,timestamp,strlen(timestamp)-1);
    strcat(prefix," :");
    write_out(out,prefix,log_buf,strlen(log_buf));
}

int main(int argc, char *argv[]) {
    fd_set	read_fds;
    int		log,net,out;/*descriptors*/
    int		ch,i;
    
    char	*logname=NULL;
    int		secoffuid=SECOFF_UID;
    int		netlisten=0;

    /*open central system log for bug errors*/
    openlog("rklogd",LOG_PID,LOG_AUTH);

    while ((ch = getopt(argc, argv, "sf:u:ln:")) != EOF)
	switch((char)ch){
            case 'f':/* Define an output file. */
                logname = optarg;
                break;
            case 'u':/* Define an secoff uid. */
                secoffuid = strtol(optarg,0,0);
                break;
	    case 'l':
		netlisten = 1;
		break;
	    case 'n':
		hostname = optarg;
		break;
	    case 's':
		use_proc = 0;
		break;
    }
    
    if (logname == NULL)  logname = getlogname(secoffuid);
			  
#ifdef TEST
    printf("secoff uid:%u\n",secoffuid);
    printf("logname:%s\n",logname);
    printf("netlisten:%u\n",netlisten);
    printf("use_proc:%u\n",use_proc);
    if (hostname!=NULL)   printf("remote:%s\n",hostname);
#endif

    
#ifndef TEST
    for (i = 1; i < NSIG; i++)	signal(i, SIG_IGN);
    signal(SIGTERM,stop_work);

    /*fork process,stay in child*/
    if ( fork() != 0 ) exit(0);

    /*we need stay in root for create pid file*/	 
    if (!check_pid(pidfile)){
	if ( mkdir(PATH_VARRUN "rklogd", 0700) && errno != EEXIST ){
	    perror("creating dir for pidfile");
	    stop_work(9);
	}
	if (chown(PATH_VARRUN "rklogd",secoffuid,0)){
	    perror("chowning dir for pidfile");
	    stop_work(9);
	}
	if ( symlink("rklogd/rklogd.pid", pidfile) ) {
	    perror("creating symlink for pidfile");
	    stop_work(9);
	}
	pidfile = PATH_VARRUN "rklogd/rklogd.pid";
	if (!write_pid(pidfile)){
	    perror("writing pidfile");
	    stop_work(9);
	}
    }else{
	    fputs("rklogd: Already running.\n", stderr);
	    stop_work(9);
    }

    /*change uid to secoff*/
    if (setuid(secoffuid)<0){
	perror("changing UID");
        exit(1);
    }

    /*child closing its file descriptors. */
    close(0);
    close(1);
    close(2);
#endif

    log=open_in();
    if (netlisten) net=open_in_net();
    
    out=open_out(logname);
    if (hostname!=NULL) open_out_net(hostname);

#ifndef TEST
    setsid(); /*finally go to daemon mode*/
#endif

    syslog(LOG_INFO, "RSBAC system logger started");

    /*I cannot use select() for syscalls*/
    if (use_proc){
	/*go to infinity loop*/
	while(1){
	    FD_ZERO(&read_fds);
	    FD_SET(log, &read_fds);
	    if (netlisten) FD_SET(net,&read_fds);

#ifdef TEST
	    puts("selecting...");
#endif

    	    if (select(20, &read_fds, NULL, NULL, NULL) >0){
		if (FD_ISSET(log,&read_fds)) read_in(log,out);
		if ((netlisten)&&(FD_ISSET(net,&read_fds))) read_net(net,out);
	    }
	}
    }else{
	/*separate thread to read messages from the network*/
	if (fork()==0){
	    pid_t  parent=getppid();
	    if (!netlisten) exit(0);
#ifdef TEST
	    puts("running second thread...");
	    printf("parent pid is %d\n",parent);
#endif
	    while(1){/*read messages from the network*/
		read_net(net,out);
		if (!(check_father(parent))) exit(1); /*father is dead*/
	    }
	}else{    
	    while(1){/*read messages from the localhost*/
		read_in(0,out);
	    }
	}
    }
}
